package com.glxp.api.admin.controller.basic;

import cn.hutool.core.io.IoUtil;
import cn.hutool.json.JSONUtil;
import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.*;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.basic.PostCorpsRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.BasicUnitMaintainExportResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysService;
import com.glxp.api.admin.service.basic.CorpImportDetailService;
import com.glxp.api.admin.service.basic.CorpImportLogService;
import com.glxp.api.admin.thread.BasicCorpImportService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.util.Date;
import java.util.List;

@Slf4j
@RestController
public class CorpImportLogController {
    @Resource
    CorpImportLogService corpImportLogService;
    @Resource
    CorpImportDetailService corpImportDetailService;
    @Resource
    BasicThirdSysService basicThirdSysService;
    @Resource
    BasicCorpImportService basicCorpImportService;

    @GetMapping("/udiwms/corps/importLog/filter")
    public BaseResponse filter(FilterUdiIpLogRequest filterUdiIpLogRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<CorpImportLogEntity> udiInfoImportLogEntities
                = corpImportLogService.filterImportLog(filterUdiIpLogRequest);
        PageInfo<CorpImportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(udiInfoImportLogEntities);
        PageSimpleResponse<CorpImportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiInfoImportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/corps/importLog/filterDetail")
    public BaseResponse filterDetail(FilterUdiIpLogRequest filterUdiIpLogRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<CorpImportDetailEntity> corpImportDetailEntities
                = corpImportDetailService.filterCorpImport(filterUdiIpLogRequest);
        PageInfo<CorpImportDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(corpImportDetailEntities);
        PageSimpleResponse<CorpImportDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(corpImportDetailEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/corps/importLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        FilterUdiIpLogRequest filterInCodeLogRequest = new FilterUdiIpLogRequest();
        filterInCodeLogRequest.setId(Integer.parseInt(id));
        List<CorpImportLogEntity> corpImportLogEntities = corpImportLogService.filterImportLog(filterInCodeLogRequest);
        if (corpImportLogEntities != null && corpImportLogEntities.size() > 0) {
            CorpImportLogEntity corpImportLogEntity = corpImportLogEntities.get(0);
            corpImportLogService.deleteById(corpImportLogEntity.getId() + "");
            corpImportDetailService.deleteByGenkey(corpImportLogEntity.getGenKey());
        }

        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udiwms/corps/importLog/delete")
    public BaseResponse delete(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (corpImportDetailService.deleteById(deleteRequest.getId()))
            return ResultVOUtils.success("删除成功");
        else
            return ResultVOUtils.error(500, "删除成功");
    }


    //往来单位信息文件导入
    @PostMapping("/udiwms/corps/importLog/upload")
    public BaseResponse uploadCorp(@RequestParam("file") List<MultipartFile> files, @RequestParam("thirdSys") String thirdSys) {
        String filePath = "D:\\udiwms\\filePath\\";
        File createFile = new File(filePath);
        if (!createFile.exists()) {
            createFile.mkdirs();
        }
        for (int i = 0; i < files.size(); i++) {
            MultipartFile file = files.get(i);
            if (file.isEmpty()) {
                return ResultVOUtils.error(500, "上传第" + (i++) + "个文件失败");
            }
            String fileName = file.getOriginalFilename();
            //解析数据
            try {
                String fileType = fileName.substring(fileName.lastIndexOf("."));
                if (".udi".equalsIgnoreCase(fileType)) {
                    String jsonData = IoUtil.read(file.getInputStream()).toString();
                    BasicUnitMaintainExportResponse data = JSONUtil.toBean(jsonData, BasicUnitMaintainExportResponse.class);

                    //导入日志
                    CorpImportLogEntity corpImportLogEntity = new CorpImportLogEntity();
                    String genKey = CustomUtil.getId();
                    corpImportLogEntity.setGenKey(genKey);
                    corpImportLogEntity.setFromType("文件导入");
                    corpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
                    corpImportLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
                    corpImportLogService.insertImportLog(corpImportLogEntity);
                    basicCorpImportService.importJsonData(data);
                    corpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
                    corpImportLogService.updateImportLog(corpImportLogEntity);
                } else {
                    return ResultVOUtils.error(500, "请上传udi文件");
                }
            } catch (Exception e) {
                log.error("导入往来单位信息错误，文件名为：{}", fileName);
            }
        }
        return ResultVOUtils.success("上传成功");
    }

    public boolean filterEmpty(Row row) {

        if (row.getCell(0) != null) {
            row.getCell(0).setCellType(CellType.STRING);
        }
        if (row.getCell(1) != null) {
            row.getCell(1).setCellType(CellType.STRING);
        }
        if (row.getCell(2) != null) {
            row.getCell(2).setCellType(CellType.STRING);
        }
        if (row.getCell(3) != null) {
            row.getCell(3).setCellType(CellType.STRING);
        }
        if (row.getCell(4) != null) {
            row.getCell(4).setCellType(CellType.STRING);
        }
        if (row.getCell(5) != null) {
            row.getCell(5).setCellType(CellType.STRING);
        }
        if (row.getCell(6) != null) {
            row.getCell(6).setCellType(CellType.STRING);
        }
        if ((row.getCell(0) == null && row.getCell(1) == null && row.getCell(2) == null
                && row.getCell(3) == null && row.getCell(4) == null && row.getCell(5) == null && row.getCell(6) == null) ||
                (
                        (row.getCell(0) != null && row.getCell(1) != null && row.getCell(2) != null
                                && row.getCell(3) != null && row.getCell(4) != null && row.getCell(5) != null && row.getCell(6) != null) &&
                                ("".equals(row.getCell(0).getStringCellValue())
                                        && "".equals(row.getCell(1).getStringCellValue())
                                        && "".equals(row.getCell(2).getStringCellValue())
                                        && "".equals(row.getCell(3).getStringCellValue())
                                        && "".equals(row.getCell(4).getStringCellValue())
                                        && "".equals(row.getCell(5).getStringCellValue())
                                        && "".equals(row.getCell(6).getStringCellValue()))
                )) {
            return true;
        }
        return false;
    }

    //往来单位信息接口上传
    @PostMapping("/udiwms/basic/corps/upload")
    public BaseResponse postUdiInfo(@RequestBody PostCorpsRequest postCorpsRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }


        String genKey = postCorpsRequest.getGenKey();
        if (genKey == null) {
            genKey = CustomUtil.getId();
        }
        CorpImportLogEntity corpImportLogEntity = corpImportLogService.selectByGenKey(genKey);
        List<CorpImportDetailEntity> corpImportDetailEntities = postCorpsRequest.getDatas();
        if (corpImportDetailEntities != null) {
            for (CorpImportDetailEntity corpImportDetailEntity : corpImportDetailEntities) {
                corpImportDetailEntity.setGenKeyFk(genKey);
                corpImportDetailEntity.setImportStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_UNPROCESS);
                corpImportDetailEntity.setUpdateTime(new java.util.Date());
                corpImportDetailService.insertCorpImport(corpImportDetailEntity);
            }
        }
        if (corpImportLogEntity == null) {
            corpImportLogEntity = new CorpImportLogEntity();
            corpImportLogEntity.setGenKey(genKey);
            corpImportLogEntity.setFromType("接口上传");
            corpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            corpImportLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
            corpImportLogService.insertImportLog(corpImportLogEntity);
        } else {
            corpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            corpImportLogService.updateImportLog(corpImportLogEntity);
        }
        basicCorpImportService.importCorps(genKey);
        return ResultVOUtils.success("上传成功");
    }

}
