package com.glxp.api.admin.thread;

import cn.hutool.core.collection.CollUtil;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.dao.basic.BasicUnitMaintainDao;
import com.glxp.api.admin.dao.thrsys.ThrCorpDao;
import com.glxp.api.admin.entity.basic.BasicUnitMaintainEntity;
import com.glxp.api.admin.entity.basic.CorpImportDetailEntity;
import com.glxp.api.admin.entity.basic.CorpImportLogEntity;
import com.glxp.api.admin.entity.thrsys.ThrCorpEntity;
import com.glxp.api.admin.req.basic.BasicUnitMaintainFilterRequest;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.res.basic.BasicUnitMaintainExportResponse;
import com.glxp.api.admin.service.basic.BasicUnitMaintainService;
import com.glxp.api.admin.service.basic.CorpImportDetailService;
import com.glxp.api.admin.service.basic.CorpImportLogService;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.TransactionIsolationLevel;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@Service
public class BasicCorpImportService {

    @Resource
    CorpImportLogService corpImportLogService;
    @Resource
    CorpImportDetailService corpImportDetailService;
    @Resource
    BasicUnitMaintainService basicUnitMaintainService;
    @Resource
    SqlSessionFactory sqlSessionFactory;

    @Async
    public void importCorps(String genKey) {

        CorpImportLogEntity corpImportLogEntity = corpImportLogService.selectByGenKey(genKey);
        if (corpImportLogEntity != null && corpImportLogEntity.getStatus() == BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS) {
            FilterUdiIpLogRequest filterUdiIpLogRequest = new FilterUdiIpLogRequest();
            filterUdiIpLogRequest.setGenKey(genKey);
            List<CorpImportDetailEntity> corpImportDetailEntities
                    = corpImportDetailService.filterCorpImport(filterUdiIpLogRequest);
            if (corpImportDetailEntities != null) {
                for (CorpImportDetailEntity corpImportDetailEntity : corpImportDetailEntities) {
                    BasicUnitMaintainEntity basicUnitMaintainEntity = new BasicUnitMaintainEntity();
                    BeanUtils.copyProperties(corpImportDetailEntity, basicUnitMaintainEntity);
                    BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest = new BasicUnitMaintainFilterRequest();
                    BeanUtils.copyProperties(corpImportDetailEntity, basicUnitMaintainFilterRequest);
                    List<BasicUnitMaintainEntity> basicUnitMaintainEntities = basicUnitMaintainService.filterList(basicUnitMaintainFilterRequest);
                    if (basicUnitMaintainEntities != null && basicUnitMaintainEntities.size() > 0) {
                        basicUnitMaintainService.insertBasicUnitMaintain(basicUnitMaintainEntity);
                        corpImportDetailEntity.setImportStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_SUCESS);
                    } else {
                        corpImportDetailEntity.setImportStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                        corpImportDetailEntity.setRemark("重复导入！");
                    }
                    corpImportDetailService.updateCorpImport(corpImportDetailEntity);

                }
            }
            corpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
            corpImportLogService.updateImportLog(corpImportLogEntity);
        }


    }

    /**
     * 导入上传文件中的往来单位信息
     *
     * @param basicUnitMaintainEntities
     */
    @Async
    public void importJsonData(BasicUnitMaintainExportResponse importData) {
        if (null != importData) {
            SqlSession batchSession = sqlSessionFactory.openSession(ExecutorType.BATCH, TransactionIsolationLevel.READ_COMMITTED);
            if (CollUtil.isNotEmpty(importData.getCorpList())) {
                try {
                    List<BasicUnitMaintainEntity> corpList = importData.getCorpList();
                    BasicUnitMaintainDao mapper = batchSession.getMapper(BasicUnitMaintainDao.class);
                    for (BasicUnitMaintainEntity basicUnitMaintainEntity : corpList) {
                        mapper.importBasicUnitMaintain(basicUnitMaintainEntity);
                    }
                    batchSession.commit();
                    log.info("导入往来单位数据成功，导入数量为：{} 条", corpList.size());
                } catch (Exception e) {
                    log.error("导入往来单位信息异常", e);
                }
            }

            if (CollUtil.isNotEmpty(importData.getThrCorpEntities())) {
                try {
                    List<ThrCorpEntity> thrCorpEntities = importData.getThrCorpEntities();
                    ThrCorpDao mapper = batchSession.getMapper(ThrCorpDao.class);
                    thrCorpEntities.forEach(thrCorpEntity -> {
                        mapper.importThrCorp(thrCorpEntity);
                    });
                    batchSession.commit();
                    log.info("导入第三方往来单位数据成功，导入数量为：{}", thrCorpEntities.size());
                } catch (Exception e) {
                    log.error("导入第三方往来单位数据失败", e);
                }
            }
        }
    }
}
